@@ -25,7 +25,7 @@ public:
   /**
    * This is a virtual constructor for the command.
    */
-  virtual cmCommand* Clone() 
+  virtual cmCommand* Clone()
     {
     return new cmElseIfCommand;
     }
@@ -45,26 +45,8 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() const { return "elseif";}
+  virtual std::string GetName() const { return "elseif";}
 
-  /**
-   * Succinct documentation.
-   */
-  virtual const char* GetTerseDocumentation() const
-    {
-    return "Starts the elseif portion of an if block.";
-    }
-  
-  /**
-   * More documentation.
-   */
-  virtual const char* GetFullDocumentation() const
-    {
-    return
-      "  elseif(expression)\n"
-      "See the if command.";
-    }
-  
   cmTypeMacro(cmElseIfCommand, cmCommand);
 };
 
